@@ -16,17 +16,17 @@ package Mail::DKIM::Algorithm::rsa_sha1;
 use base "Mail::DKIM::Algorithm::Base";
 use Carp;
 use MIME::Base64;
-use Digest::SHA1;
+use Digest::SHA;
 
 sub init_digests
 {
 	my $self = shift;
 
 	# initialize a SHA-1 Digest
-	$self->{header_digest} = new Digest::SHA1;
+	$self->{header_digest} = Digest::SHA->new(1);
 	if ($self->{draft_version} eq "01")
 	{
-		$self->{body_digest} = new Digest::SHA1;
+		$self->{body_digest} = Digest::SHA->new(1);
 	}
 	else
 	{
@@ -60,26 +60,9 @@ sub verify
 	return $self->check_body_hash;
 }
 
+sub wants_pre_signature_headers
+{
+	return 1;
+}
+
 1;
-
-__END__
-
-=head1 NAME
-
-Mail::DKIM::Algorithm::rsa_sha1 - implements the rsa-sha1 signing algorithm for DKIM
-
-=head1 CONSTRUCTOR
-
-=head2 new() - create an object for the DKIM signing algorithm "rsa-sha1"
-
-  my $algorithm = new Mail::DKIM::Algorithm::rsa_sha1(
-                      Signature => $dkim_signature
-                  );
-
-=head1 METHODS
-
-See Mail::DKIM::Algorithm::Base for the full list of methods supported
-by this algorithm class.
-
-=cut
-
