@@ -15,7 +15,7 @@ use Mail::DKIM::Signature;
 package Mail::DKIM::Common;
 use base "Mail::DKIM::MessageParser";
 use Carp;
-our $VERSION = 0.31;
+our $VERSION = 0.33;
 
 sub new
 {
@@ -138,7 +138,7 @@ sub message_originator
 	if ($self->{headers_by_name}->{from})
 	{
 		my @list = Mail::Address->parse($self->{headers_by_name}->{from});
-		return $list[0];
+		return $list[0] if @list;
 	}
 	return Mail::Address->new;
 }
@@ -151,12 +151,12 @@ sub message_sender
 	if ($self->{headers_by_name}->{sender})
 	{
 		my @list = Mail::Address->parse($self->{headers_by_name}->{sender});
-		return $list[0];
+		return $list[0] if @list;
 	}
 	if ($self->{headers_by_name}->{from})
 	{
 		my @list = Mail::Address->parse($self->{headers_by_name}->{from});
-		return $list[0];
+		return $list[0] if @list;
 	}
 	return Mail::Address->new;
 }
