@@ -102,15 +102,6 @@ the DomainKey-Signature that gets prepended to a signed message.
 
 =cut
 
-sub as_string
-{
-	my $self = shift;
-
-	my $prefix = $self->{prefix} || "DomainKey-Signature:";
-
-	return $prefix . $self->Mail::DKIM::KeyValueList::as_string;
-}
-
 sub as_string_without_data
 {
 	croak "as_string_without_data not implemented";
@@ -171,6 +162,11 @@ sub canonicalization
 	return lc($self->get_tag("c")) || "simple";
 }	
 
+sub DEFAULT_PREFIX
+{
+	return "DomainKey-Signature:";
+}
+
 =head2 domain() - get or set the domain (d=) field
 
   my $d = $signature->domain;          # gets the domain value
@@ -274,11 +270,54 @@ In list context, the header field names will be returned as a list.
 #sub headerlist
 # is in Signature.pm
 
+=head2 identity() - get the signing identity
+
+  my $i = $signature->identity;
+
+In DomainKey signatures, the signing identity is the first address
+found in the Sender header or the From header. This field is
+populated by the Verifier when processing a DomainKey signature.
+
+=cut
+
 sub identity
 {
 	my $self = shift;
 	croak "cannot change identity on " . ref($self) if @_;
-	return "@" . $self->domain;
+	return $self->{dk_identity};
+}
+
+=head2 identity_source() - determine which header had the identity
+
+  my $source = $signature->identity_source;
+
+If the message is being verified, this method will tell you which
+of the message headers was used to determine the signature identity.
+Possible values are "header.sender" and "header.from".
+
+=cut
+
+sub identity_source
+{
+	my $self = shift;
+	croak "unexpected argument" if @_;
+	return $self->{dk_identity_source};
+}
+
+# init_identity() - initialize the DomainKeys concept of identity
+#
+# The signing identity of a DomainKeys signature is the sender
+# of the message itself, i.e. the address in the Sender/From header.
+# The sender may not be known when the signature object is
+# constructed (since the signature usually precedes the From/Sender
+# header), so use this method when you have the From/Sender value.
+# See also finish_header() in Mail::DKIM::Verifier.
+#
+sub init_identity
+{
+	my $self = shift;
+	$self->{dk_identity} = shift;
+	$self->{dk_identity_source} = shift;
 }
 
 sub method
