@@ -47,7 +47,7 @@ sub parse
 		my ($tagname, $value) = split(/\s*=\s*/, $raw_tag, 2);
 		unless (defined $value)
 		{
-			die "key value list syntax error\n";
+			die "syntax error\n";
 		}
 
 		$tag->{name} = $tagname;
@@ -160,16 +160,24 @@ sub wrap
 	{
 		my $tagname = $tag->{name};
 		my $tagtype = $args{Tags}->{$tagname} || $args{Default} || "";
+
+		$wrap->{Break} = undef;
+		$wrap->{BreakBefore} = undef;
+		$did_first ? $wrap->add(";") : ($did_first = 1);
+
 		my ($raw_name, $raw_value) = split(/=/, $tag->{raw}, 2);
 		unless ($args{PreserveNames})
 		{
+			$wrap->flush; #allow a break before the tag name
 			$raw_name =~ s/^\s*/ /;
 			$raw_name =~ s/\s+$//;
 		}
+		$wrap->add($raw_name . "=");
 
 		if ($tagtype eq "b64")
 		{
 			$raw_value =~ s/\s+//gs;   #removes all whitespace
+			$wrap->flush;
 			$wrap->{Break} = qr/./;
 		}
 		elsif ($tagtype eq "list")
@@ -177,21 +185,18 @@ sub wrap
 			$raw_value =~ s/\s+/ /gs;   #reduces any whitespace to single space
 			$raw_value =~ s/^\s|\s$//g; #trims preceding/trailing spaces
 			$raw_value =~ s/\s*:\s*/:/g;
-			$wrap->{Break} = qr/[\s:]/;
+			$wrap->flush;
+			$wrap->{Break} = qr/[\s]/;
+			$wrap->{BreakBefore} = qr/[:]/;
 		}
 		elsif ($tagtype eq "")
 		{
 			$raw_value =~ s/\s+/ /gs;   #reduces any whitespace to single space
 			$raw_value =~ s/^\s|\s$//g; #trims preceding/trailing spaces
+			$wrap->flush;
 			$wrap->{Break} = qr/\s/;
 		}
-		else
-		{
-			$wrap->{Break} = undef;
-		}
-
-		$did_first ? $wrap->add(";") : ($did_first = 1);
-		$wrap->add($raw_name . "=" . $raw_value);
+		$wrap->add($raw_value);
 	}
 
 	$wrap->finish;
