@@ -15,6 +15,8 @@ package Mail::DKIM::PublicKey;
 use base ("Mail::DKIM::KeyValueList", "Mail::DKIM::Key");
 *calculate_EM = \&Mail::DKIM::Key::calculate_EM;
 
+use Mail::DKIM::DNS;
+
 sub new {
 	my $type = shift;
 	my %prms = @_;
@@ -30,10 +32,15 @@ sub new {
 	bless $self, $type;
 }
 
+# my $public_key = Mail::DKIM::PublicKey->fetch(
+#                     Protocol => "dns",
+#                   );
+# Protocol: from the q= tag of the signature, usually "dns"
+# Selector: from the s= tag of the signature
+# Domain: from the d= tag of the signature
+#
 sub fetch
 {
-	use Net::DNS;
-
 	my $class = shift;
 	my %prms = @_;
 
@@ -48,37 +55,18 @@ sub fetch
 
 	my $host = $prms{'Selector'} . "._domainkey." . $prms{'Domain'};
 
-	my $rslv = new Net::DNS::Resolver or
-		return;
-	
 	#
 	# perform DNS query for public key...
 	#   if the query takes too long, we should generate an error
 	#
-	my $resp;
-	eval
-	{
-		# set a 10 second timeout
-		local $SIG{ALRM} = sub { die "DNS query timeout for $host\n" };
-		alarm 10;
-
-		$resp = $rslv->query($host, "TXT");
-		alarm 0;
-	};
-	my $E = $@;
-	alarm 0; #FIXME- restore previous alarm?
-	if ($E)
-	{
-		chomp $E;
-		die "$E\n";
-	}
-	unless ($resp)
+	my @resp = Mail::DKIM::DNS::query($host, "TXT");
+	unless (@resp)
 	{
 		# no response => NXDOMAIN
 		return;
 	}
 
-	foreach my $ans ($resp->answer) {
+	foreach my $ans (@resp) {
 		next unless $ans->type eq "TXT";
 		$strn = join "", $ans->char_str_list;
 	}
@@ -104,7 +92,7 @@ sub check
 	{
 		unless ($v eq "DKIM1")
 		{
-			die "unrecognized public key version\n";
+			die "unsupported version\n";
 		}
 	}
 
@@ -116,7 +104,7 @@ sub check
 	{
 		unless ($k eq "rsa")
 		{
-			die "public key: unsupported key type\n";
+			die "unsupported key type\n";
 		}
 	}
 
@@ -124,15 +112,15 @@ sub check
 	my $p = $self->data;
 	if (not defined $p)
 	{
-		die "public key: missing p= tag\n";
+		die "missing p= tag\n";
 	}
 	if ($p eq "")
 	{
-		die "public key: revoked\n";
+		die "revoked\n";
 	}
 	unless ($p =~ /^[A-Za-z0-9\+\/\=]+$/)
 	{
-		die "public key: invalid data\n";
+		die "invalid data\n";
 	}
 
 	# have OpenSSL load the key
@@ -146,11 +134,11 @@ sub check
 		chomp (my $E = $@);
 		if ($E =~ /(OpenSSL error: .*?) at /)
 		{
-			$E = "public key: $1";
+			$E = "$1";
 		}
 		elsif ($E =~ /^(panic:.*?) at /)
 		{
-			$E = "public key: OpenSSL $1";
+			$E = "OpenSSL $1";
 		}
 		die "$E\n";
 	}
@@ -161,13 +149,102 @@ sub check
 		my @list = split(/:/, $s);
 		unless (grep { $_ eq "*" || $_ eq "email" } @list)
 		{
-			die "public key: does not support email\n";
+			die "does not support email\n";
 		}
 	}
 
 	return 1;
 }
 
+# check_granularity() - check whether this key matches signature identity
+# 
+# a public key record can restrict what identities it may sign with,
+#   g=, granularity, restricts the local part of the identity
+#   t=s, restricts whether subdomains can be used
+#
+# This method returns true if the given identity is allowed by this
+# public key; it returns false otherwise.
+# If false is returned, you can check C<$@> for an explanation of
+# why.
+#
+sub check_granularity
+{
+	my $self = shift;
+	my ($identity, $empty_g_means_wildcard) = @_;
+
+	# check granularity
+	my $g = $self->granularity;
+
+	# yuck- what is this $empty_g_means_wildcard parameter?
+	# well, it turns out that with DomainKeys signatures,
+	# an empty g= is the same as g=*
+	if ($g eq "" && $empty_g_means_wildcard)
+	{
+		$g = "*";
+	}
+
+	# split i= value into a "local part" and a "domain part"
+	my ($local_part, $domain_part);
+	if ($identity =~ /^(.*)\@([^@]*)$/)
+	{
+		$local_part = $1;
+		$domain_part = $2;
+	}
+	else
+	{
+		$local_part = "";
+		$domain_part = $identity;
+	}
+
+
+	my ($begins, $ends) = split /\*/, $g, 2;
+	if (defined $ends)
+	{
+		# the g= tag contains an asterisk
+
+		# the local part must be at least as long as the pattern
+		if (length($local_part) < length($begins) + length($ends)
+			or
+		# the local part must begin with $begins
+			substr($local_part, 0, length($begins)) ne $begins
+			or
+		# the local part must end with $ends
+			(length($ends) && substr($local_part, -length($ends)) ne $ends))
+		{
+			$@ = "granularity mismatch\n";
+			return;
+		}
+	}
+	else
+	{
+		if ($g eq "")
+		{
+			$@ = "granularity is empty\n";
+			return;
+		}
+		unless ($local_part eq $begins)
+		{
+			$@ = "granularity mismatch\n";
+			return;
+		}
+	}
+
+	# check subdomains
+	if ($self->subdomain_flag)
+	{
+		unless ($domain_part eq lc($self->{'Domain'}))
+		{
+			$@ = "does not support signing subdomains\n";
+			return;
+		}
+	}
+
+	return 1;
+}
+
+# returns true if the actual hash algorithm used is listed by this
+# public key; dies otherwise
+#
 sub check_hash_algorithm
 {
 	my $self = shift;
@@ -179,12 +256,15 @@ sub check_hash_algorithm
 		my @list = split(/:/, $h);
 		unless (grep { $_ eq $hash_algorithm } @list)
 		{
-			die "public key: does not support hash algorithm '$hash_algorithm'\n";
+			die "does not support hash algorithm '$hash_algorithm'\n";
 		}
 	}
 	return 1;
 }
 
+# Create an OpenSSL public key object from the Base64-encoded data
+# found in this public key's DNS record. The OpenSSL object is saved
+# in the "cork" property.
 sub convert
 {
 	use Crypt::OpenSSL::RSA;
@@ -235,14 +315,39 @@ sub verify {
 	return $rtrn;
 }
 
+=head2 granularity() - get or set the granularity (g=) field
+
+  my $g = $public_key->granularity;
+
+  $public_key->granularity("*");
+
+Granularity of the key. The value must match the Local-part of the
+effective "i=" tag of the DKIM-Signature header field.
+The granularity is a literal value, or a pattern with a single '*'
+wildcard character that matches zero or more characters.
+
+If no granularity is defined, then the default value, '*', will
+be returned.
+
+=cut
+
 sub granularity
 {
 	my $self = shift;
 
+	# set new granularity if provided
 	(@_) and 
 		$self->set_tag("g", shift);
 
-	return $self->get_tag("g");
+	my $g = $self->get_tag("g");
+	if (defined $g)
+	{
+		return $g;
+	}
+	else
+	{
+		return '*';
+	}
 }
 
 sub notes
@@ -275,7 +380,18 @@ sub flags
 	(@_) and 
 		$self->set_tag("t", shift);
 
-	return $self->get_tag("t");
+	return $self->get_tag("t") || "";
+}
+
+# subdomain_flag() - checks whether "s" is specified in flags
+#
+# returns true if "s" is found, false otherwise
+#
+sub subdomain_flag
+{
+	my $self = shift;
+	my @flags = split /:/, $self->flags;
+	return grep { $_ eq "s" } @flags;
 }
 
 sub revoked
@@ -308,6 +424,10 @@ sub verify_sha1_digest
 	return $self->verify_digest("SHA-1", $digest, $signature);
 }
 
+# verify_digest() - returns true if the digest verifies, false otherwise
+#
+# if false, $@ is set to a description of the problem
+#
 sub verify_digest
 {
 	my $self = shift;
